
from tempfile import NamedTemporaryFile
from unittest import TestCase

from ch2.commands.args import bootstrap_file, m, V
from ch2.commands.kit import start, change, statistics, finish, show, undo
from ch2.config import default
from ch2.squeal import KitModel, KitItem, KitComponent


class TestKit(TestCase):

    def test_bikes(self):
        with NamedTemporaryFile() as f:
            args, db = bootstrap_file(f, m(V), '5', configurator=default)
            with db.session_context() as s:
                with self.assertRaises(Exception) as ctx:
                    start(s, 'bike', 'cotic', None, False)
                self.assertTrue('--force' in str(ctx.exception), ctx.exception)
                start(s, 'bike', 'cotic', None, True)
                # run('sqlite3 %s ".dump"' % f.name, shell=True)
                with self.assertRaises(Exception) as ctx:
                    start(s, 'xxxx', 'marin', None, False)
                self.assertTrue('--force' in str(ctx.exception), ctx.exception)
                start(s, 'bike', 'marin', None, False)
                with self.assertRaises(Exception) as ctx:
                    change(s, 'cotic', 'chain', 'sram', '2018-02-01', False, False)
                self.assertTrue('--force' in str(ctx.exception))
                change(s, 'cotic', 'chain', 'sram', None, True, True)
                change(s, 'cotic', 'chain', 'kcm', '2018-01-01', False, False)
                change(s, 'cotic', 'chain', 'sram', '2018-05-01', False, False)
                change(s, 'cotic', 'chain', 'kcm', '2018-07-01', False, False)
                change(s, 'cotic', 'chain', 'sram', '2018-04-01', False, False)
                with self.assertRaises(Exception) as ctx:
                    start(s, 'bike', 'bike', None, True)
                self.assertTrue('bike' in str(ctx.exception), ctx.exception)
                with self.assertRaises(Exception) as ctx:
                    start(s, 'bike', 'sram', None, True)
                self.assertTrue('sram' in str(ctx.exception), ctx.exception)
                start(s, 'bike', 'bowman', None, False)
                change(s, 'bowman', 'chain', 'sram', None, False, True)
                self.assertEqual(len(show(s, 'cotic', None)), 3)
                self.assertEqual(len(statistics(s, 'bike')), 16)
                self.assertEqual(len(statistics(s, 'cotic')), 20)
                self.assertEqual(len(statistics(s, 'chain')), 33)
                self.assertEqual(len(statistics(s, 'sram')), 16)
                finish(s, 'bowman', None, False)
                with self.assertRaises(Exception) as ctx:
                    finish(s, 'bowman', None, False)
                self.assertTrue('retired' in str(ctx.exception), ctx.exception)
                self.assertEqual(len(KitModel.get_all(s, KitItem.get(s, 'cotic'), KitComponent.get(s, 'chain'))), 5)
                undo(s, 'cotic', 'chain', 'sram', None, True)
                self.assertEqual(len(KitModel.get_all(s, KitItem.get(s, 'cotic'), KitComponent.get(s, 'chain'))), 2)
                undo(s, 'cotic', 'chain', 'kcm', None, True)
                self.assertEqual(len(KitModel.get_all(s, KitItem.get(s, 'cotic'), KitComponent.get(s, 'chain'))), 0)
                undo(s, 'bowman', 'chain', 'sram', None, True)
                self.assertFalse(KitComponent.get(s, 'chain', require=False))




from logging import getLogger

from . import MultiProcCalculator, ActivityJournalCalculatorMixin
from ...squeal import StatisticJournal
from ...squeal.tables.kit import expand_item

log = getLogger(__name__)


class KitCalculator(ActivityJournalCalculatorMixin, MultiProcCalculator):

    def _run_one(self, s, time):
        ajournal = self._get_source(s, time)
        kit = StatisticJournal.for_source(s, ajournal.id, 'kit', self.owner_in, ajournal.activity_group)
        if kit:
            log.debug(f'Read {kit.value} at {time} / {ajournal.activity_group.name}')
            for kit_name in kit.value.split(','):
                for kit_instance in expand_item(s, kit_name, ajournal.start):
                    kit_instance.add_use(s, ajournal.start, source=ajournal, owner=self.owner_out)
                    log.debug(f'Added usage for {kit_instance}')
        else:
            log.debug(f'No kit defined for this activity ({time})')




from bokeh import tile_providers
from bokeh.io import output_notebook
from bokeh.layouts import row, column
from bokeh.models import PreText, Slider
from bokeh.plotting import show, figure

from ch2.data import *
from ch2.lib import *
from ch2.squeal import Segment
from ch2.uranus.decorator import template


@template
def define_segment(local_time, activity_group_name):

    f'''
    # Define Segment: {local_time.split()[0]}
    '''

    '''
    $contents
    '''

    '''
    ## Load Data
    
    Open a connection to the database and load the data for a ride containing the segment.
    '''
    s = session('-v2')
    df = activity_statistics(s, SPHERICAL_MERCATOR_X, SPHERICAL_MERCATOR_Y, DISTANCE, LATITUDE, LONGITUDE,
                             local_time=local_time, activity_group_name=activity_group_name)
    df.dropna(inplace=True)

    '''
    ### Select Segment

    Use the sliders to isolate the segment.
    
    You may need to play with both sliders before the map displays correctly 
    (todo - fix this, and also all the log messages).  
    '''
    CARTODBPOSITRON = tile_providers.get_provider(tile_providers.Vendors.CARTODBPOSITRON)

    output_notebook()
    width, height = 800, 800

    def modify_doc(doc):

        t1 = PreText(height=20, width=200)
        t2 = PreText(height=20, width=200)
        t3 = PreText(height=20, width=100)
        s1 = Slider(start=0, end=len(df), value=0, title='Start')
        s2 = Slider(start=1, end=len(df)-1, value=len(df), title='Length')
        f = figure(plot_width=width, plot_height=height, x_axis_type='mercator', y_axis_type='mercator')
        c = column(row(s1, s2), row(t1, t2, t3), f)

        def mkplot(l1, l2):
            l2 = min(len(df)-1, l1+l2)
            t1.text = '%9.5f,%9.5f' % (df.iloc[l1]['Longitude'], df.iloc[l1]['Latitude'])
            t2.text = '%9.5f,%9.5f' % (df.iloc[l2]['Longitude'], df.iloc[l2]['Latitude'])
            t3.text = '%4.1fkm' % ((df.iloc[l2]['Distance'] - df.iloc[l1]['Distance']) / 1000)
            f = figure(plot_width=width, plot_height=height, x_axis_type='mercator', y_axis_type='mercator')
            f.add_tile(CARTODBPOSITRON)
            f.circle(x='Spherical Mercator X', y='Spherical Mercator Y', source=df[l1:l2])
            c.children[2] = f

        s1.on_change('value', lambda attr, old, new: mkplot(s1.value, s2.value))
        s2.on_change('value', lambda attr, old, new: mkplot(s1.value, s2.value))
        doc.add_root(c)

    show(modify_doc)

    '''
    ## Define Segment
    
    Replace the details below with your own from the plot above.
    The start and finish values are (lat, lon); the distance is in metres.
    
    Finally, uncomment the `s.add()` to add this to the database.
    '''

    activity_group = ActivityGroup.from_name(s, 'Bike')
    segment = Segment(start=(-70.61813,-33.41536), finish=(-70.63340,-33.42655), distance=4400,
                      activity_group=activity_group,
                      name='San Cristobal', description='Climb up San Cristobal in Parque Metropolitana')
    #s.add(x)
